/*######################################################################
#
# Project: DoD Configuration Extension
# URL: http://www.nabber.org/projects/
# E-mail: webmaster@nabber.org
#
# Copyright: (C) 2003-2006, Neil McNab
# License: GNU General Public License Version 2
#   (http://www.gnu.org/copyleft/gpl.html)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Filename: options.js
# Author(s): Neil McNab
# Last Updated: Thursday, January 4, 2007
#
######################################################################*/

function dod_initialize_options() {
	var count = 0;
	var myvalue = {};

	//grab the objects we need to use from XUL
	dod_domain = document.getElementById("dodDomain");

	dod_enable_domain_highlight = document.getElementById("dodEnableDomainHighlight");
	enable_cac = document.getElementById("dodEnableCACAlert");
	dod_enable_redirect = document.getElementById("dodEnableRedirect");

	dod_forged_warning = document.getElementById("dodForgedAlert");
	dod_security_level = document.getElementById("dod_security_listbox");
	require_class = document.getElementById("dodRequireClassification");
	dod_classifications = document.getElementById("dod_classifications");
	dod_subject = document.getElementById("dodSubjectClassification");

	children_list = get_array("extensions.dod.classification.");

	if (isBrowser()) {
		//populate options interface for browser
		document.getElementById("dod_browser_options").hidden = false;
		
	}

	if (isMail()) {
		//populate options interface for mail
		document.getElementById("dod_mail_options").hidden = false;

		var previous = "";

		for (i = 0; i < children_list.length; i++) {
			var component_list = children_list[i].split(".");
			var security_domain = component_list[0];
			var level = component_list[1];
			var root = component_list[2];

			var label = prefManager.getCharPref("extensions.dod.classification." + children_list[i]);

			if (security_domain != previous) {
				// create a menu for each security domain
				var menu = document.createElement("menulist");

				menu.setAttribute("id", "dod_list_" + security_domain);
				menu.addEventListener('command', update_defaults_list, false);
				menu.label = security_domain;

				var popup = document.createElement("menupopup")
				menu.appendChild(popup);

				//add none selection
				var myitem = document.createElement("menuitem");
				myitem.setAttribute("label", security_domain + " (none)");
				myitem.setAttribute("value", 0);

				popup.appendChild(myitem);

				dod_classifications.appendChild(menu);
				previous = security_domain;
			}
			if (root == 0) {
				// add an item to the menu for each major security domain classification
				var myitem = document.createElement("menuitem");
				label = label.split(";")[0];
				myitem.setAttribute("label", security_domain + " " + label);
				myitem.setAttribute("value", level);
				popup.appendChild(myitem);
			}
		}


	}

	//initialize values from preferences
        dod_domain.value = prefManager.getCharPref("extensions.dod.domain");
	dod_enable_domain_highlight.checked = prefManager.getBoolPref("extensions.dod.enable_domain_highlight");
	enable_cac.checked = prefManager.getBoolPref("extensions.dod.enable_cac_alert");
	dod_enable_redirect.checked = prefManager.getBoolPref("extensions.dod.enable_https_redirect");

	dod_forged_warning.checked = prefManager.getBoolPref("extensions.dod.forged_warning");
	dod_security_level.value = prefManager.getIntPref("extensions.dod.security_level");
	require_class.checked = prefManager.getBoolPref("extensions.dod.require_classification");
	dod_subject.checked = prefManager.getBoolPref("extensions.dod.subject_classification");

	if (isMail()) {
		children_list = get_array("extensions.dod.max_level.");
		for (i = 0; i < children_list.length; i++) {
			var security_domain = children_list[i];
			var domain_box = document.getElementById("dod_list_" + security_domain);
			domain_box.setAttribute("value", prefManager.getIntPref("extensions.dod.max_level." + security_domain));
		}
		dod_listbox = update_defaults_list();
	}
}

function dod_save_options() {

	// save selections to preferences
        prefManager.setCharPref("extensions.dod.domain", dod_domain.value);
	prefManager.setBoolPref("extensions.dod.enable_domain_highlight", dod_enable_domain_highlight.checked);
	prefManager.setBoolPref("extensions.dod.enable_cac_alert", enable_cac.checked);
	prefManager.setBoolPref("extensions.dod.enable_https_redirect", dod_enable_redirect.checked);

	prefManager.setBoolPref("extensions.dod.forged_warning", dod_forged_warning.checked);
	prefManager.setIntPref("extensions.dod.security_level", dod_security_level.value);
	prefManager.setBoolPref("extensions.dod.require_classification", require_class.checked);
	prefManager.setBoolPref("extensions.dod.subject_classification", dod_subject.checked);

	if (isMail()) {
		children_list = get_array("extensions.dod.max_level.");
		for (i = 0; i < children_list.length; i++) {
			var security_domain = children_list[i];
			var domain_box = document.getElementById("dod_list_" + security_domain);
			prefManager.setIntPref("extensions.dod.max_level." + security_domain, domain_box.value);
		}
		//alert(dod_listbox.value);
		prefManager.setCharPref("extensions.dod.default_classification", dod_listbox.value);
		dod_update_security_level();
	}
}

function update_defaults_list() {
	// populate the available classifications based on the levels selected
	name = "dod_default_listbox";
        allow_none = true;

	children_list = get_array("extensions.dod.classification.");

	var dod_listbox = document.getElementById(name);
	dod_listbox.removeAllItems();
	dod_listbox.hidden = false;

	for (i = 0; i < children_list.length; i++) {
		var component_list = children_list[i].split(".");
		var security_domain = component_list[0];
		var level = component_list[1];

		var label = prefManager.getCharPref("extensions.dod.classification." + children_list[i]);
		var menu = document.getElementById("dod_list_" + security_domain);

		if (menu.value >= level) {
			label = label.split(";")[0];
			dod_listbox.appendItem(security_domain + " " + label, children_list[i]);
		}
	}
	if (allow_none) {
		dod_listbox.appendItem("(None)", "");
	}
	
	dod_listbox.value = prefManager.getCharPref("extensions.dod.default_classification");

	return dod_listbox;
}