/*######################################################################
#
# Project: DoD Configuration Extension
# URL: http://www.nabber.org/projects/
# E-mail: webmaster@nabber.org
#
# Copyright: (C) 2003-2006, Neil McNab
# License: GNU General Public License Version 2
#   (http://www.gnu.org/copyleft/gpl.html)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Filename: utils.js
# Author(s): Neil McNab
# Last Updated: Saturday, January 11, 2007
#
######################################################################*/

//global utilities go in here

const DOD_CURRENT_VER=62;
const DOD_TITLE="DoD Configuration Extension";
const LDAP_NAME="DoD 411";
const LDAP_URI="ldap://dod411.gds.disa.mil:389/ou=DoD, o=U.S. Government,c=us??sub?(objectclass=*)";

var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);


function file_download(url, localfile) {
	var request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIJSXMLHttpRequest);
	request.open("GET", url, true);
	// required for proper handling of binary files
	request.overrideMimeType('text/plain; charset=x-user-defined');
	request.onreadystatechange = function (aEvt) {
		if (request.readyState == 4) {
			if(request.status == 200) {
				var filepath = writeFile(localfile, request.responseText);
				load_cert(filepath);
			} else {
				alert("Could not download certificate from " + url);
			}
		}
	};
	request.send(null);
}

function load_cert(filepath) {
	var certdb = Components.classes["@mozilla.org/security/x509certdb;1"].getService(Components.interfaces.nsIX509CertDB);

    	var filename = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    	filename.initWithPath(filepath);

	try {
		certdb.importCertsFromFile(null, filename, Components.interfaces.nsIX509Cert.CA_CERT);
	} catch (e) {};

}

function writeFile( sFilePath, sFileContent )
{
	//create proper path
	const id = "{d15c1608-ba3e-4aa0-aa6f-aa9337226087}";
	var ext = Components.classes["@mozilla.org/extensions/manager;1"]
                    .getService(Components.interfaces.nsIExtensionManager)
                    .getInstallLocation(id)
                    .getItemLocation(id);
	ext.append("content");
	ext.append("temp");
	ext.append(sFilePath);

	try {

		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.QueryInterface(Components.interfaces.nsIFile);
		file.initWithPath(ext.path);

		// overwrite file if it already exists
		if( file.exists() == true ) file.remove( false );

		var strm = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		strm.QueryInterface(Components.interfaces.nsIOutputStream);
		strm.QueryInterface(Components.interfaces.nsISeekableStream);
		strm.init( file, 0x04 | 0x08, 420, 0);
		strm.write( sFileContent, sFileContent.length );
		strm.flush();
		strm.close();
	} catch(ex) { }
	return ext.path;
}

function isBrowser() {
	if (prefManager.getPrefType("browser.chromeURL") != 0) {
		return true;
	}
	return false;
}

function isMail() {
	if (prefManager.getPrefType("mail.remember_password") != 0) {
		return true;
	}
	return false;
}

function trusted_domain(check_string) {
	domain = prefManager.getCharPref("extensions.dod.domain");
	domain = domain.toLowerCase();
	check_string = check_string.toLowerCase();

	if (domain[0] != ".") {
		domain = "." + domain;
	}

	if (check_string.lastIndexOf(domain) == (check_string.length - domain.length)) {
		return true;
	}
	return false;
}

function get_array(branch_base) {
	var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	var branch = prefs.getBranch(branch_base);
	var children = branch.getChildList("", {});
	var children_list = String(children).split(",");
	children_list = children_list.sort();

	return children_list;
}

function read_stream(inputStream, count){
	var stream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
	stream.setInputStream(inputStream);

	var c;
	var csi;
	var h = "";
			
	// PARSE THE HEADERS
	for (csi = 0; csi < count; csi++) {
		c = String.fromCharCode(stream.read8());
		h += c;
	}

	// handle continuation lines
	h = h.replace(/(\r\n\s)/g, "");

	// return array of lines
	return h.split("\r\n");
}

function dod_update_security_level() {
	var security_level = prefManager.getIntPref("extensions.dod.security_level");

	if (security_level <= 50) {
		//lowest security level, minimum settings apply
	    	//turn off javascript
    		prefManager.setBoolPref("javascript.allow.mailnews", false);

	    	//disable remote image loading
    		prefManager.setBoolPref("mailnews.message_display.disable_remote_image", true);

	}
	if (security_level <= 40) {
		//loop through all open windows
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator(null);

		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			try {
				
			} catch (e) {}
		}

		// disable HTML email composing
		var ids = get_array("mail.identity.id");
		var prev = "";
		for (i = 0; i < ids.length; i++) {
			var id = ids[i];
			var parts = id.split("\.");
			id = parts[0];
			if (id != prev) {
				prefManager.setBoolPref("mail.identity.id" + id + ".compose_html", "false");
			}
			prev = id;
		}
		prefManager.setBoolPref("mail.html_compose", "false");
		prefManager.setBoolPref("mail.identity.default.compose_html", "false");

		//disable HTML viewing, set to plain text
		prefManager.setIntPref("mailnews.display.html_as", "1");
		
	}
	if (security_level <= 30) {
		// disable preview pane
		//loop through all open windows
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator(null);
		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			var splitter = win.document.getElementById("threadpane-splitter");
			try {
				splitter.setAttribute("state", "collapsed");
			} catch (e) {}
		}  
	}
	if (security_level <= 20) {
	
	}
	if (security_level <= 10) {
		//highest security level
	}

}
