/*######################################################################
#
# Project: DoD Configuration Extension
# URL: http://www.nabber.org/projects/
# E-mail: webmaster@nabber.org
#
# Copyright: (C) 2003-2006, Neil McNab
# License: GNU General Public License Version 2
#   (http://www.gnu.org/copyleft/gpl.html)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Filename: viewpane.js
# Author(s): Neil McNab
# Last Updated: Friday, January 19, 2007
#
######################################################################*/

// Whenever the messagepane loads, update info
const IPRegEx = /^.*\[([\d\.]+)\].*/;

var messagepane = document.getElementById("messagepane");
messagepane.addEventListener("load", getclassificationheader, true);
messagepane.addEventListener("load", getdomainwarning, true);

function dodupdatestatus(in_domain) {
	// update the message pane with what we know
	var expandeddomainwarningBox = document.getElementById("expandeddomainwarningBox");

	if (in_domain > 0) {
		expandeddomainwarningBox.headerValue = "THIS MESSAGE DOES NOT ORIGINATE FROM THE " + prefManager.getCharPref("extensions.dod.domain").toUpperCase() + " DOMAIN!  It could be forged if no digital signature is present.";
		expandeddomainwarningBox.hidden = false;

	} else if (in_domain < 0) {
		expandeddomainwarningBox.headerValue = "Could not verify that this message originated from within the " + prefManager.getCharPref("extensions.dod.domain") + " network.  You might be working offline.";
		expandeddomainwarningBox.hidden = false;

	} else if (in_domain == 0) {
		expandeddomainwarningBox.hidden = true;
	}
};

function getdomainwarning() {
	// reset to not display anything (message is OK)
	dodupdatestatus(0);
	// kick out of function, we aren't checking the message
    	if (!prefManager.getBoolPref("extensions.dod.forged_warning")) {
        	return;
    	}
    	var uri = GetFirstSelectedMessage();

    try {
 	var msgService = messenger.messageServiceFromURI(uri);
    } catch (e) {};

    var dataListener = {
		stream : null,
		h : "",
		host: "",
		in_domain: 0,

		handledns: function(data) {

			data = String(data);

			// no need to check the same server twice in a row
			if (this.host == data) {
				return;
			}

			this.host = data;

			// ignore localhost, that doesn't help us determine origin
			if (this.host == "localhost") {
				return;
			}

			// unable to determine, no hostname for reverse lookup
			if ((this.host == "") || ("null" == this.host)) {
				this.in_domain = -1;
				dodupdatestatus(this.in_domain);
				return;
			}
			
			// finally, check is a host is in our domain, if not, flag warning
			if (!trusted_domain(this.host)) {
				this.in_domain = 2;
				dodupdatestatus(this.in_domain);
				return;
			}

			
		},

		onStartRequest: function(request, context) {},
		onStopRequest: function(request, context, status) { },
		onDataAvailable: function(request, context, inputStream, offset, count) {

			this.in_domain = 0;
			dodupdatestatus(this.in_domain);

			var lines = read_stream(inputStream, count);
			var myheader = "Received: ";

			for (i = 0; i < lines.length; i++) {
				// check for matching header
				if (lines[i].substr(0, myheader.length) == myheader) {
					var parts = IPRegEx.exec(lines[i]);
					try {
						var ip = parts[1];
					} catch (e) {}

					// if offline, this will still error out, need to fix
					try {
						queryDNS(DNS_ReverseIPHostname(ip), "PTR", this.handledns);	
					} catch (e) {
						if (this.in_domain <= 0) {
							this.in_domain = -1;
							dodupdatestatus(this.in_domain);
						}
					}
				
				}
			}
		
		}
    };

	var async_consumer = Components.classes["@mozilla.org/network/async-stream-listener;1"].createInstance();
	var async_consumer2 = async_consumer.QueryInterface(Components.interfaces.nsIAsyncStreamListener);

	async_consumer2.init(dataListener, null);

	try {
		msgService.streamMessage(uri, async_consumer, msgWindow, null, false, null)
	} catch (ex) {
		return;
	}
	
}

function getclassificationheader() {
    var uri = GetFirstSelectedMessage();

    try {
    	var msgService = messenger.messageServiceFromURI(uri);
    } catch (e) {};
	
    var dataListener = {
		stream : null,
		h : "",
		mode : 0, // 0=Looking for first Received:, 1=Looking for second Received:
		hcont : false,
		hlast : "",
		bytesread : 0,
		
		onStartRequest: function(request, context) {},
		onStopRequest: function(request, context, status) { },
		onDataAvailable: function(request, context, inputStream, offset, count) {

			var lines = read_stream(inputStream, count);

			// handle header
			var myheader = "X-Security-Classification: ";

			for (i = 0; i < lines.length; i++) {
				// check if header matches
				if (lines[i].substr(0, myheader.length) == myheader) {
					var temp = lines[i].substr(myheader.length).split(";");
					expandedclassificationBox.headerValue = temp[0] + " (" + temp[1] + ")";
					expandedclassificationBox.hidden = false;
				}
			}
		}
    };

	var async_consumer = Components.classes["@mozilla.org/network/async-stream-listener;1"].createInstance();
	var async_consumer2 = async_consumer.QueryInterface(Components.interfaces.nsIAsyncStreamListener);

	async_consumer2.init(dataListener, null);

	try {
		msgService.streamMessage(uri, async_consumer, msgWindow, null, false, null)
	} catch (ex) {
		return;
	}
	
}
